﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;

namespace KAT_Browser
{
    class KATSearchPage
    {
        private string _pageContent, _searchTerm;
        private Torrent[] _torrents = new Torrent[] { };
        public int PageNumber { get; private set; }
        public string NextPageLink { get; private set; }
        public string PreviousPageLink { get; private set; }

        public void LoadPage(string pageContent)
        {
            _pageContent = pageContent;
            _pageContent = _pageContent.Replace("\r", string.Empty);
            _pageContent = _pageContent.Replace("\n", string.Empty);
            SetTorrents();
            SetCurrentPage(); 
            SetSearchTerm();
            NextPageLink = GetPageLink(PageNumber + 1);
            PreviousPageLink = GetPageLink(PageNumber - 1);
        }

        #region Setup
        private string GetPageLink(int page)
        {
            string FORMAT = @"""(?<URL>/usearch/\w+/{0}/)\"" class=\""turnoverButton siteButton bigButton\""\>\<span\>{0}\</span\>";
            string pattern = String.Format(FORMAT, page);
            Regex regex = new Regex(pattern, RegexOptions.IgnoreCase | RegexOptions.ExplicitCapture);
            Match match = regex.Match(_pageContent);

            if (match.Success)
            {
                return @"http://kat.ph" + match.Groups["URL"].Value;
            }
            else
            {
                FORMAT = @"http://kat.ph/usearch/{0}/";
                return String.Format(FORMAT, _searchTerm);
            }
        }

        private void SetCurrentPage()
        {
            //Page 398 of 400
            //const string PATTERN = @"page (?<Page>\d+) of \d+";
            const string PATTERN = @"siteButton bigButton active""\>\<span\>(?<Page>\d+)\</span\>";
            Regex regex = new Regex(PATTERN, RegexOptions.IgnoreCase | RegexOptions.ExplicitCapture);
            Match match = regex.Match(_pageContent);

            if (match.Success)
            {
                string temp = match.Groups["Page"].Value;
                int pageNum = 0;
                int.TryParse(temp, out pageNum);
                PageNumber = pageNum;
            }
            else
            {
                Debug.WriteLine("Page num parse fail");
                PageNumber = 1;
            }
        }

        private void SetSearchTerm()
        {
            const string PATTERN = @"http://kat\.ph/usearch/(?<Term>\w+)/\?rss=\d+";
            Match match = Regex.Match(_pageContent, PATTERN);
            if (match.Success) _searchTerm = match.Groups["Term"].Value;
        }

        private void SetTorrents()
        {
            string TORRENT_PATTERN = @"\<tr class=""(odd|even)"" id="".+?""\>.+?\</tr\>";
            Regex regex = new Regex(TORRENT_PATTERN, RegexOptions.Singleline | RegexOptions.ExplicitCapture);
            MatchCollection MC = regex.Matches(_pageContent);
            List<Torrent> torrentList = new List<Torrent>();

            foreach (Match match in MC)
            {
                torrentList.Add(new Torrent(match.Value));
            }

            _torrents = torrentList.ToArray();
        }
        #endregion

        /// <summary>
        /// Determines whether or not the page has torrents. Use this to see
        /// if a page object needs updating
        /// </summary>
        public bool HasTorrents
        {
            get { return (_torrents != null && _torrents.Length > 0); }
        }

        /// <summary>
        /// Gets the downloaded html content of the webpage
        /// </summary>
        public string Content
        {
            get { return _pageContent; }
        }

        public string SearchTerm
        {
            get { return _searchTerm; }
        }

        public Torrent[] Torrents
        {
            get { return _torrents; }
        }
    }
}
