﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Drawing;
using System.IO;
using System.ComponentModel;

namespace BSFControls.ExtendedControls
{
    class PathPickLabel : Label
    {
        private string _path;
        private bool _showFullPath = true;
        private Color _mouseLeaveColor = Color.FromKnownColor(KnownColor.Control);
        private Color _mouseEnterColor = Color.FromKnownColor(KnownColor.ControlLight);

        public PathPickLabel()
        {
            InitialText = "...";
            this.AutoEllipsis = true;
            this.BorderStyle = BorderStyle.FixedSingle;
            this.TextAlign = ContentAlignment.MiddleCenter;
            this.Cursor = Cursors.Hand;
        }

        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [DefaultValue(false)]
        public override bool AutoSize
        {
            get
            {
                return base.AutoSize;
            }
            set
            {
                base.AutoSize = value;
            }
        }

        protected override void InitLayout()
        {
            base.InitLayout();
            base.AutoSize = false;
        }

        protected override void OnMouseEnter(EventArgs e)
        {
            base.OnMouseEnter(e);
            this.BackColor = _mouseEnterColor;
        }

        protected override void OnMouseLeave(EventArgs e)
        {
            base.OnMouseLeave(e);
            this.BackColor = _mouseLeaveColor;
        }

        public Color MouseLeaveColor
        {
            get { return _mouseLeaveColor; }
            set 
            {
                _mouseLeaveColor = value;
                this.BackColor = value;
            }
        }

        public Color MouseEnterColor
        {
            get { return _mouseEnterColor; }
            set { _mouseEnterColor = value; }
        }

        /// <summary>
        /// Determines whether or not to show the full path or the filename with the extension.
        /// Note: Only for filepaths
        /// </summary>
        public bool ShowFullPath
        {
            get { return _showFullPath; }
            set { _showFullPath = value; }
        }

        /// <summary>
        /// Gets or sets the full path of the directory or filename
        /// </summary>
        public string FullPath
        {
            get { return _path; }
            set 
            {
                _path = value;

                if (_showFullPath)
                {
                    this.Text = _path;
                }
                else
                {
                    this.Text = Path.GetFileName(_path);
                }
            }
        }

        public override string Text
        {
            get
            {
                return base.Text;
            }
            set
            {
                if (string.IsNullOrEmpty(_path))
                {
                    base.Text = InitialText;
                }
                else
                {
                    base.Text = value;
                }
            }
        }

        public string InitialText { get; set; }
    }
}
