﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;

namespace KAT_Browser
{
    public class Torrent
    {
        private string _section;
        private float _sizeInMB;
        private string _downloadLink;

        /// <summary>
        /// Gets whether GB is the desired unit of measurement for this torrent
        /// </summary>
        public bool UseGB { get; set; }

        public Torrent(string section)
        {
            _section = section.Replace("\r", string.Empty);
            _section = section.Replace("\n", string.Empty);
            SetDownloadSize();
            SetDownloadLink();
        }

        private void SetDownloadLink()
        {
            // <a title="Download torrent file" href="//torcache.net/torrevdscr.xvid.ac3.hq.hive.cm8"
            Regex regex = new Regex(@"\<a title=""Download torrent file"" href=""(?<Link>.+?)""");
            Match match = regex.Match(_section);

            if (match.Success)
            {
                _downloadLink = match.Groups["Link"].Value;
            }
        }

        private void SetDownloadSize()
        {
            // <td class="nobr center">2.36 <span>GB</span></td>
            const string PATTERN = @"\<td class=""nobr center""\>(?<Size>[\d\. ]+?)\<span\>(?<Unit>GB|MB)\<";
            Regex regex = new Regex(PATTERN, RegexOptions.IgnoreCase | RegexOptions.Singleline);
            Match match = regex.Match(_section);

            if (match.Success)
            {
                string temp = match.Groups["Size"].Value.Trim();
                float num = 0;
                bool success = float.TryParse(temp, out num);

                if (success)
                {
                    string unitOfMeasure = match.Groups["Unit"].Value;

                    // We can get the size in two forms, MB and GB
                    if (unitOfMeasure == "MB")
                    {
                        _sizeInMB = num;
                        UseGB = false;
                    }
                    else
                    {
                        _sizeInMB = num * 1000f;
                        UseGB = true;
                    }
                }
            }
        }

        /// <summary>
        /// Properly capitalizes common acronyms
        /// </summary>
        private static string FixAcronyms(string name)
        {
            const string PATTERN = 
            @"(\b) 
            (\d\d[a-z][a-z]|[a-z]\d[a-z]|[a-z][a-z]\d)|             (?# Two letters with one number)
            (dvd|(rip|scr|r)|ts|avi|mkv|acc|aac|hdtv|hq|hd(|rip))|  (?# Common acronyms)
            (S\d+E\d+)                                              (?# TV Shows)
            (\b)";

            RegexOptions options = RegexOptions.IgnoreCase | RegexOptions.IgnorePatternWhitespace 
                                                           | RegexOptions.ExplicitCapture;

            MatchCollection MC = Regex.Matches(name, PATTERN, options);

            foreach (Match match in MC)
            {
                string temp = name.Substring(match.Index, match.Length);
                temp = temp.ToUpper();
                int len = name.Length;
                name = name.Remove(match.Index, match.Length);
                name = name.Insert(match.Index, temp); 
            }

            return name;
        }

        private static string CapatilizeAllWords(string text)
        {
            TextInfo textInfo = new CultureInfo("en-US", false).TextInfo;
            return textInfo.ToTitleCase(text);
        }

        #region Properties
        public string Name
        {
            get
            {
                // title=[kat.ph]dr.bombay.the.hits.cd.2007.sds
                Regex regex = new Regex(@"title=\[kat\.ph\](?<Title>.+?)""", RegexOptions.IgnoreCase);
                Match match = regex.Match(_section);

                if (match.Success)
                {
                    string name = match.Groups["Title"].Value;
                    name = name.Replace('.', ' ');
                    name = FixAcronyms(name);
                    return CapatilizeAllWords(name);
                }

                return string.Empty;
            }
        }

        public int Seeds
        {
            get
            {
                // <td class="green center">14042</td>
                Regex regex = new Regex(@"\<td class=""green center""\>(?<Seeds>\d+)\</td\>");
                Match match = regex.Match(_section);

                if (match.Success)
                {
                    string temp = match.Groups["Seeds"].Value;
                    int num = 0;
                    bool success = int.TryParse(temp, out num);
                    if (success) return num;
                }

                return -1;
            }
        }

        public int Leeches
        {
            get
            {
                //<td class="red lasttd center">8308</td>
                Regex regex = new Regex(@"\<td class=""red lasttd center""\>(?<Leeches>\d+)\</td\>");
                Match match = regex.Match(_section);

                if (match.Success)
                {
                    string temp = match.Groups["Leeches"].Value;
                    int num = 0;
                    bool success = int.TryParse(temp, out num);
                    if (success) return num;
                }

                return -1;
            }
        }

        public float SizeInMB
        {
            get { return _sizeInMB; }
        }

        public float SizeInGB
        {
            get { return _sizeInMB / 1000; }
        }

        public string SizeString
        {
            get
            {
                if (UseGB) return SizeInGB.ToString() + " GB";
                else return _sizeInMB.ToString() + " MB";
            }
        }

        public string DownloadLink
        {
            get { return _downloadLink; }
        }
        #endregion
    }
}
