﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Diagnostics;
using KAT_Browser.Properties;
using System.IO;

namespace KAT_Browser
{
    public partial class TorrentDisplay : UserControl
    {
        public Color MouseLeaveColor { get; set; }
        public Color MouseEnterColor { get; set; }
        public Torrent Torrent { get; private set; }

        public TorrentDisplay(Torrent torrent)
        {
            InitializeComponent();
            MouseEnterColor = Color.FromArgb(202, 81, 0);
            MouseLeaveColor = Color.FromKnownColor(KnownColor.Control);
            this.Torrent = torrent;
            lblTorrentName.Text = torrent.Name;

            int seedCount = torrent.Seeds;
            if (seedCount == -1) lblSeeds.Text += "N/A";
            else lblSeeds.Text = seedCount.ToString();

            int leechCount = torrent.Leeches;
            if (leechCount == -1) lblLeeches.Text += "N/A";
            else lblLeeches.Text = leechCount.ToString();

            btnDownload.Tag = torrent.DownloadLink;

            lblSize.Text = torrent.SizeString;
        }

        private string GetTorrentDestDir()
        {
            string destDir = Settings.Default.TorrentDest.TrimEnd('\\') + "\\";

            if (destDir.Length > 2)
            {
                if (!Directory.Exists(destDir))
                {
                    try
                    {
                        Directory.CreateDirectory(destDir);
                    }
                    catch { return string.Empty; }
                }

                return destDir;
            }
            else
            {
                return string.Empty;
            }
        }

        private void btnDownload_Click(object sender, EventArgs e)
        {
            TorrentDownloader downloader = new TorrentDownloader();
            TorrentDownloaderParams ps = new TorrentDownloaderParams();

            if (Settings.Default.TorrentDest.Length > 0)
            {
                ps.Destination = GetTorrentDestDir() + GetFileName();
            }
            else
            {
                ps.Destination = GetFileName();
            }

            ps.OpenWhenFinished = Settings.Default.OpenTorrents;
            ps.URL = "http:" + btnDownload.Tag.ToString();
            downloader.RunWorkerAsync(ps);
        }

        private string GetFileName()
        {
            string fileName = MakeStringFilenameSafe(lblTorrentName.Text);

            if (fileName.Length > 40)
            {
                fileName = fileName.Substring(0, 40);
            }

            return fileName.Trim() + ".torrent";
        }

        /// <summary>
        /// Determines if a cher is has one of the characters in a specified char array
        /// </summary>
        private static bool ContainsChar(char input, params char[] patterns)
        {
            for (int i = 0; i < patterns.Length; i++)
            {
                if (patterns[i].Equals(input))
                {
                    return true;
                }
            }

            return false;
        }

        private static bool IsCharFileNameSafe(char c)
        {
            char[] safeSymbols = { '-', ' ', '\b', '_', '.' };
            bool isAcceptedSymbol = ContainsChar(c, safeSymbols);

            // if character is alphanumeric and is a safe symbol return true
            return (char.IsLetterOrDigit(c) || isAcceptedSymbol);
        }

        private static string MakeStringFilenameSafe(string text)
        {
            StringBuilder builder = new StringBuilder();

            for (int i = 0; i < text.Length; i++)
            {
                if (IsCharFileNameSafe(text[i]))
                {
                    builder.Append(text[i]);
                }
            }

            return builder.ToString();
        }

        private void lblTorrentName_Click(object sender, EventArgs e)
        {
            this.OnClick(e);
            this.Focus(); // Make panel scrollable by mouse wheel
        }

        private void lblSize_MouseEnter(object sender, EventArgs e)
        {
            this.BackColor = MouseEnterColor;
        }

        private void lblSize_MouseLeave(object sender, EventArgs e)
        {
            this.BackColor = MouseLeaveColor;
        }
    }
}
