﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using System.IO.Compression;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace KAT_Browser
{
    class WebPageDownloader : BackgroundWorker
    {
        private string _result = null;
        private Exception _ex;

        protected override void OnDoWork(DoWorkEventArgs e)
        {
            base.OnDoWork(e);
            _ex = null;
            _result = null;
            string URL = e.Argument.ToString();
            WebRequest req = WebRequest.Create(URL);
            req.Timeout = 15 * 1000;
            Stream stream = null;

            try
            {
                WebResponse response = req.GetResponse();
                stream = response.GetResponseStream();

                if (response.Headers.AllKeys.Contains("Content-Encoding")
                    && response.Headers["Content-Encoding"].Contains("gzip"))
                {
                    stream = new GZipStream(stream, CompressionMode.Decompress);
                }

                using (StreamReader reader = new StreamReader(stream))
                {
                    _result = reader.ReadToEnd();
                }
            }
            catch (Exception ex) { _ex = ex; }
            finally
            {
                if (stream != null) stream.Dispose();
            }
        }

        protected override void OnRunWorkerCompleted(RunWorkerCompletedEventArgs e)
        {
            e = new RunWorkerCompletedEventArgs(_result, _ex, e.Cancelled);
            base.OnRunWorkerCompleted(e);
        }
    }
}
